@@ -92,7 +92,7 @@ static uint32 get_bits(FILE *f, int n, struct bit_buffer *bb)
 	}
 
 	while (bb->count < 24) {
-		bb->buffer |= read8(f) << bb->count;
+		bb->buffer |= read8(f, NULL) << bb->count;
 		bb->count += 8;
 	}
 
@@ -113,7 +113,7 @@ static void block_copy(struct block *block, struct sub_block *sub,
 	}
 }
 
-static void block_unpack_16bit(struct block *block, struct sub_block *sub,
+static int block_unpack_16bit(struct block *block, struct sub_block *sub,
 			       FILE *in, FILE *out)
 {
 	struct bit_buffer bb;
@@ -124,8 +124,12 @@ static void block_unpack_16bit(struct block *block, struct sub_block *sub,
 	bb.count = 0;
 	bb.buffer = 0;
 
-	fseek(out, sub->unpk_pos, SEEK_SET);
-	fseek(in, block->tt_entries, SEEK_SET);
+	if (fseek(out, sub->unpk_pos, SEEK_SET) < 0) {
+		return -1;
+	}
+	if (fseek(in, block->tt_entries, SEEK_SET) < 0) {
+		return -1;
+	}
 
 	for (j = 0; j < block->sub_blk; ) {
 		uint32 size = sub[j].unpk_size >> 1;
@@ -175,12 +179,16 @@ static void block_unpack_16bit(struct block *block, struct sub_block *sub,
 				break;
 
 			pos = 0;
-			fseek(out, sub[j].unpk_pos, SEEK_SET);
+			if (fseek(out, sub[j].unpk_pos, SEEK_SET) < 0) {
+				return -1;
+			}
 		}
 	}
+
+	return 0;
 }
 
-static void block_unpack_8bit(struct block *block, struct sub_block *sub,
+static int block_unpack_8bit(struct block *block, struct sub_block *sub,
 			      FILE *in, FILE *out)
 {
 	struct bit_buffer bb;
@@ -189,13 +197,19 @@ static void block_unpack_8bit(struct block *block, struct sub_block *sub,
 	uint32 j, oldval = 0;
 	uint8 ptable[0x100];
 
-	fread(ptable, 1, 0x100, in);
+	if (fread(ptable, 1, 0x100, in) != 0x100) {
+		return -1;
+	}
 
 	bb.count = 0;
 	bb.buffer = 0;
 
-	fseek(out, sub->unpk_pos, SEEK_SET);
-	fseek(in, block->tt_entries, SEEK_SET);
+	if (fseek(out, sub->unpk_pos, SEEK_SET) < 0) {
+		return -1;
+	}
+	if (fseek(in, block->tt_entries, SEEK_SET) < 0) {
+		return -1;
+	}
 
 	for (j = 0; j < block->sub_blk; ) {
 		uint32 size = sub[j].unpk_size;
@@ -236,11 +250,15 @@ static void block_unpack_8bit(struct block *block, struct sub_block *sub,
 		if (pos >= size) {
 			if (++j >= block->sub_blk)
 				break;
-;
+
 			pos = 0;
-			fseek(out, sub[j].unpk_pos, SEEK_SET);
+			if (fseek(out, sub[j].unpk_pos, SEEK_SET) < 0) {
+				return -1;
+			}
 		}
 	}
+
+	return 0;
 }
 
 static int test_mmcmp(unsigned char *b)
@@ -253,48 +271,67 @@ static int decrunch_mmcmp(FILE *in, FILE *out)
 	struct header h;
 	uint32 *table;
 	uint32 i, j;
+	int error;
 
 	/* Read file header */
-	if (read32l(in) != 0x4352697A)		/* ziRC */
+	if (read32l(in, NULL) != 0x4352697A)		/* ziRC */
 		goto err;
-	if (read32l(in) != 0x61694e4f)		/* ONia */
+	if (read32l(in, NULL) != 0x61694e4f)		/* ONia */
 		goto err;
-	if (read16l(in) < 14)			/* header size */
+	if (read16l(in, NULL) < 14)			/* header size */
 		goto err;
 
 	/* Read header */
-	h.version = read16l(in);
-	h.nblocks = read16l(in);
-	h.filesize = read32l(in);
-	h.blktable = read32l(in);
-	h.glb_comp = read8(in);
-	h.fmt_comp = read8(in);
+	h.version = read16l(in, &error);
+	if (error != 0) goto err;
+	h.nblocks = read16l(in, &error);
+	if (error != 0) goto err;
+	h.filesize = read32l(in, &error);
+	if (error != 0) goto err;
+	h.blktable = read32l(in, &error);
+	if (error != 0) goto err;
+	h.glb_comp = read8(in, &error);
+	if (error != 0) goto err;
+	h.fmt_comp = read8(in, &error);
+	if (error != 0) goto err;
 
 	if (h.nblocks == 0)
 		goto err;
 
 	/* Block table */
-	fseek(in, h.blktable, SEEK_SET);
-	table = malloc(h.nblocks * 4);
-	if (table == NULL)
+	if (fseek(in, h.blktable, SEEK_SET) < 0) {
 		goto err;
+	}
+
+	if ((table = malloc(h.nblocks * 4)) == NULL) {
+		goto err;
+	}
 
 	for (i = 0; i < h.nblocks; i++) {
-		table[i] = read32l(in);
+		table[i] = read32l(in, &error);
+		if (error != 0) goto err2;
 	}
 
 	for (i = 0; i < h.nblocks; i++) {
 		struct block block;
 		struct sub_block *sub_block;
+		uint8 buf[20];
 
-		fseek(in, table[i], SEEK_SET);
-		block.unpk_size  = read32l(in);
-		block.pk_size    = read32l(in);
-		block.xor_chk    = read32l(in);
-		block.sub_blk    = read16l(in);
-		block.flags      = read16l(in);
-		block.tt_entries = read16l(in);
-		block.num_bits   = read16l(in);
+		if (fseek(in, table[i], SEEK_SET) < 0) {
+			goto err2;
+		}
+
+		if (fread(buf, 1, 20, in) != 20) {
+			goto err2;
+		}
+
+		block.unpk_size  = readmem32l(buf);
+		block.pk_size    = readmem32l(buf + 4);
+		block.xor_chk    = readmem32l(buf + 8);
+		block.sub_blk    = readmem16l(buf + 12);
+		block.flags      = readmem16l(buf + 14);
+		block.tt_entries = readmem16l(buf + 16);
+		block.num_bits   = readmem16l(buf + 18);
 
                 /* Sanity check */
 		if (block.unpk_size <= 0 || block.pk_size <= 0)
@@ -320,8 +357,15 @@ static int decrunch_mmcmp(FILE *in, FILE *out)
 			goto err2;
 
 		for (j = 0; j < block.sub_blk; j++) {
-			sub_block[j].unpk_pos  = read32l(in);
-			sub_block[j].unpk_size = read32l(in);
+			uint8 buf[8];
+
+			if (fread(buf, 1, 8, in) != 8) {
+				free(sub_block);
+				goto err2;
+			}
+
+			sub_block[j].unpk_pos  = readmem32l(buf);
+			sub_block[j].unpk_size = readmem32l(buf + 4);
 
 	                /* Sanity check */
 			if (sub_block[j].unpk_pos < 0 ||
@@ -338,10 +382,16 @@ static int decrunch_mmcmp(FILE *in, FILE *out)
 			block_copy(&block, sub_block, in, out);
 		} else if (block.flags & MMCMP_16BIT) {
 			/* Data is 16-bit packed */
-			block_unpack_16bit(&block, sub_block, in, out);
+			if (block_unpack_16bit(&block, sub_block, in, out) < 0) {
+				free(sub_block);
+				goto err2;
+			}
 		} else {
 			/* Data is 8-bit packed */
-			block_unpack_8bit(&block, sub_block, in, out);
+			if (block_unpack_8bit(&block, sub_block, in, out) < 0) {
+				free(sub_block);
+				goto err2;
+			}
 		}
 
 		free(sub_block);
