@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2015 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU Lesser General Public License. See COPYING.LIB
@@ -29,10 +29,11 @@ static int decrunch_muse(FILE *f, FILE *fo)
 {                                                          
 	uint32 checksum;
   
-	fseek(f, 24, SEEK_SET);
-	inflate(f, fo, &checksum, 0);
+	if (fseek(f, 24, SEEK_SET) < 0) {
+		return -1;
+	}
 
-	return 0;
+	return inflate(f, fo, &checksum, 0);
 }
 
 struct depacker muse_depacker = {
