@@ -1,9 +1,23 @@
 /* Extended Module Player
- * Copyright (C) 1996-2015 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr
  *
- * This file is part of the Extended Module Player and is distributed
- * under the terms of the GNU Lesser General Public License. See COPYING.LIB
- * for more information.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+ * THE SOFTWARE.
  */
 
 #include <stdio.h>
@@ -29,37 +43,56 @@ int test_oxm(FILE *f)
 	uint32 ilen;
 	int slen[256];
 	uint8 buf[1024];
+	int error;
 
-	fseek(f, 0, SEEK_SET);
-	if (fread(buf, 1, 16, f) < 16)
+	if (fseek(f, 0, SEEK_SET) < 0) {
 		return -1;
-	if (memcmp(buf, "Extended Module:", 16))
-		return -1;
-	
-	fseek(f, 60, SEEK_SET);
-	hlen = read32l(f);
-	fseek(f, 6, SEEK_CUR);
-	npat = read16l(f);
-	nins = read16l(f);
+	}
 
-	if (npat > 256 || nins > 128)
+	if (fread(buf, 1, 80, f) != 80) {
 		return -1;
+	}
+
+	hlen = readmem32l(buf + 60);
+	npat = readmem16l(buf + 70);
+	nins = readmem16l(buf + 72);
+
+	if (memcmp(buf, "Extended Module:", 16)) {
+		return -1;
+	}
 	
-	fseek(f, 60 + hlen, SEEK_SET);
+	if (npat > 256 || nins > 128) {
+		return -1;
+	}
+	
+	if (fseek(f, 60 + hlen, SEEK_SET) < 0) {
+		return -1;
+	}
 
 	for (i = 0; i < npat; i++) {
-		len = read32l(f);
-		fseek(f, 3, SEEK_CUR);
-		plen = read16l(f);
-		fseek(f, len - 9 + plen, SEEK_CUR);
+		if (fread(buf, 1, 9, f) != 9) {
+			return -1;
+		}
+		len = readmem32l(buf);
+		plen = readmem16l(buf + 7);
+
+		if (fseek(f, len - 9 + plen, SEEK_CUR) < 0) {
+			return -1;
+		}
 	}
 
 	for (i = 0; i < nins; i++) {
-		ilen = read32l(f);
-		if (ilen > 263)
+		ilen = read32l(f, &error);
+		if (error != 0) return -1;
+		if (ilen > 263) {
 			return -1;
-		fseek(f, -4, SEEK_CUR);
-		fread(buf, ilen, 1, f);		/* instrument header */
+		}
+		if (fseek(f, -4, SEEK_CUR) < 0) {
+			return -1;
+		}
+		if (fread(buf, 1, ilen, f) != ilen) { /* instrument header */
+			return -1;
+		}
 		nsmp = readmem16l(buf + 27);
 
 		if (nsmp > 255)
@@ -69,16 +102,22 @@ int test_oxm(FILE *f)
 
 		/* Read instrument data */
 		for (j = 0; j < nsmp; j++) {
-			slen[j] = read32l(f);
-			fseek(f, 36, SEEK_CUR);
+			slen[j] = read32l(f, &error);
+			if (error != 0) {
+				return -1;
+			}
+			if (fseek(f, 36, SEEK_CUR) < 0) {
+				return -1;
+			}
 		}
 
 		/* Read samples */
 		for (j = 0; j < nsmp; j++) {
-			read32b(f);
-			if (read32b(f) == MAGIC_OGGS)
+			read32b(f, NULL);
+			if (read32b(f, NULL) == MAGIC_OGGS)
 				return 0;
-			fseek(f, slen[j] - 8, SEEK_CUR);
+			if (fseek(f, slen[j] - 8, SEEK_CUR) < 0)
+				return -1;
 		}
 	}
 
@@ -92,21 +131,28 @@ static char *oggdec(FILE *f, int len, int res, int *newlen)
 	uint8 *data, *pcm;
 	int16 *pcm16 = NULL;
 	uint32 id;
+	int error;
 
 	/* Sanity check */
 	if (len < 4) {
 		return NULL;
 	}
 
-	/*size =*/ read32l(f);
-	id = read32b(f);
-	fseek(f, -8, SEEK_CUR);
+	/*size =*/ read32l(f, &error);
+	if (error != 0)
+		return NULL;
+	id = read32b(f, &error);
+	if (error != 0 || fseek(f, -8, SEEK_CUR) < 0)
+		return NULL;
 
 	if ((data = calloc(1, len)) == NULL)
 		return NULL;
 
-	read32b(f);
-	fread(data, 1, len - 4, f);
+	read32b(f, &error);
+	if (error != 0 || fread(data, 1, len - 4, f) != len - 4) {
+		free(data);
+		return NULL;
+	}
 
 	if (id != MAGIC_OGGS) {		/* copy input data if not Ogg file */
 		*newlen = len;
@@ -161,33 +207,55 @@ static int decrunch_oxm(FILE *f, FILE *fo)
 	char *pcm[256];
 	int newlen = 0;
 
-	fseek(f, 60, SEEK_SET);
-	hlen = read32l(f);
-	fseek(f, 6, SEEK_CUR);
-	npat = read16l(f);
-	nins = read16l(f);
-	
-	fseek(f, 60 + hlen, SEEK_SET);
+	if (fread(buf, 1, 80, f) != 80) {
+		return -1;
+	}
+
+	hlen = readmem32l(buf + 60);
+	npat = readmem16l(buf + 70);
+	nins = readmem16l(buf + 72);
+
+	if (npat > 256 || nins > 128) {
+		return -1;
+	}
+
+	if (fseek(f, 60 + hlen, SEEK_SET) < 0) {
+		return -1;
+	}
 
 	for (i = 0; i < npat; i++) {
-		len = read32l(f);
-		fseek(f, 3, SEEK_CUR);
-		plen = read16l(f);
-		fseek(f, len - 9 + plen, SEEK_CUR);
+		if (fread(buf, 1, 9, f) != 9) {
+			return -1;
+		}
+		len = readmem32l(buf);
+		plen = readmem16l(buf + 7);
+
+		if (fseek(f, len - 9 + plen, SEEK_CUR) < 0) {
+			return -1;
+		}
 	}
 
 	pos = ftell(f);
-	fseek(f, 0, SEEK_SET);
+	if (pos < 0) {
+		return -1;
+	}
+	if (fseek(f, 0, SEEK_SET) < 0) {
+		return -1;
+	}
 	move_data(fo, f, pos);			/* module header + patterns */
 
 	for (i = 0; i < nins; i++) {
-		ilen = read32l(f);
+		ilen = read32l(f, NULL);
 		if (ilen > 1024) {
 			D_(D_CRIT "ilen=%d\n", ilen);
 			return -1;
 		}
-		fseek(f, -4, SEEK_CUR);
-		fread(buf, ilen, 1, f);		/* instrument header */
+		if (fseek(f, -4, SEEK_CUR) < 0) {
+			return -1;
+		}
+		if (fread(buf, ilen, 1, f) != 1) {	/* instrument header */
+			return -1;
+		}
 		buf[26] = 0;
 		fwrite(buf, ilen, 1, fo);
 		nsmp = readmem16l(buf + 27);
@@ -205,12 +273,14 @@ static int decrunch_oxm(FILE *f, FILE *fo)
 
 		/* Read sample headers */
 		for (j = 0; j < nsmp; j++) {
-			xi[j].len = read32l(f);
+			xi[j].len = read32l(f, NULL);
 			if (xi[j].len > MAX_SAMPLE_SIZE) {
 				D_(D_CRIT "sample %d len = %d", j, xi[j].len);
 				return -1;
 			}
-			fread(xi[j].buf, 1, 36, f);
+			if (fread(xi[j].buf, 1, 36, f) != 36) {
+				return -1;
+			}
 		}
 
 		/* Read samples */
