@@ -26,7 +26,7 @@
 #include "common.h"
 #include "depacker.h"
 
-#define val(p) ((p)[0]<<16 | (p)[1] << 8 | (p)[2])
+/* #define val(p) ((p)[0]<<16 | (p)[1] << 8 | (p)[2]) */
 
 
 static int savefile(FILE *fo, void *mem, size_t length)
@@ -145,7 +145,7 @@ static int ppdepack(uint8 *data, size_t len, FILE *fo)
     return -1;
   }
 
-  outlen = (data[len-4]<<16) | (data[len-3]<<8) | data[len-2];
+  outlen = readmem24b(data + len - 4);
 
   /* fprintf(stderr, "decrunched length = %u bytes\n", outlen); */
 
@@ -184,7 +184,9 @@ static int decrunch_pp(FILE *f, FILE *fo)
     if (fo == NULL)
         goto err;
 
-    fstat(fileno(f), &st);
+    if (fstat(fileno(f), &st) < 0)
+	goto err;
+
     plen = st.st_size;
     //counter = 0;
 
@@ -206,7 +208,9 @@ static int decrunch_pp(FILE *f, FILE *fo)
 	 goto err;
     }
 
-    fread (packed, plen, 1, f);
+    if (fread(packed, 1, plen, f) != plen) {
+         goto err1;
+    }
 
     /* Hmmh... original pp20 only support efficiency from 9 9 9 9 up to 9 10 12 13, afaik
      * but the xfd detection code says this... *sigh*
@@ -224,12 +228,12 @@ static int decrunch_pp(FILE *f, FILE *fo)
     }
 
 
-    if (((((val (packed +4) ) * 256 ) + packed[7] ) & 0xf0f0f0f0) != 0 ) {
+    if (((readmem24b(packed +4)  * 256  + packed[7]) & 0xf0f0f0f0) != 0 ) {
 	 /*fprintf(stderr, "invalid efficiency(?)\n");*/
          goto err1;
     }
 
-    unplen = val (packed + plen - 4);
+    unplen = readmem24b(packed + plen - 4);
     if (!unplen) {
 	 /*fprintf(stderr, "not a powerpacked file\n");*/
          goto err1;
