@@ -271,6 +271,11 @@ static int make_table(struct LhADecrData *dat, int16 nchar, uint8 bitlen[], int1
   for(j = 0; j < nchar; j++)
   {
     k = bitlen[j];
+
+    /* Sanity check */
+    if(k >= 17)
+      return -1;
+
     if(k == 0)
       continue;
     l = start[k] + weight[k];
@@ -288,8 +293,18 @@ static int make_table(struct LhADecrData *dat, int16 nchar, uint8 bitlen[], int1
     }
     else
     {
+#if 0
+      /* CID 156018 (#1 of 1): Logically dead code (DEADCODE)
+       * dead_error_line: Execution cannot reach this statement: return -1;
+       */
+      /* Sanity check */
+      if(k >= 17)
+        return -1;
+#endif
+
       /* code not in table */
-      p = &table[(i = start[k]) >> m];
+      i = start[k];
+      p = &table[i >> m];
       i <<= tablebits;
       n = k - tablebits;
       /* make tree (n length) */
@@ -1466,7 +1481,11 @@ static int32 LhA_Decrunch(FILE *in, FILE *out, int size, uint32 Method)
 
       dicsiz = 1 << dd->DicBit;
 
+#ifdef ENABLE_LARC
       offset = (Method == LARC_METHOD || Method == PMARC2_METHOD) ? 0x100 - 2 : 0x100 - 3;
+#else
+      offset = 0x100 - 3;
+#endif
 
       if((text = dd->text = calloc(dicsiz, 1)))
       {
@@ -1476,9 +1495,7 @@ static int32 LhA_Decrunch(FILE *in, FILE *out, int size, uint32 Method)
           memset(text, ' ', (size_t) dicsiz);
 
         if (DecodeStart(dd) < 0) {
-          free(dd->text);
-          free(dd);
-          return -1;
+          goto error;
         }
 
         --dicsiz; /* now used with AND */
@@ -1489,9 +1506,7 @@ static int32 LhA_Decrunch(FILE *in, FILE *out, int size, uint32 Method)
 
           c = DecodeC(dd);
           if (c < 0) {
-            free(dd->text);
-            free(dd);
-            return -1;  
+            goto error;
           }
 
 	  if (dd->error)
@@ -1499,7 +1514,11 @@ static int32 LhA_Decrunch(FILE *in, FILE *out, int size, uint32 Method)
 	
           if(c <= UCHAR_MAX)
           {
-            text[dd->loc++] = fputc(c, out);
+            int res = fputc(c, out);
+            if (res < 0) {
+              goto error;
+            }
+            text[dd->loc++] = res;
             dd->loc &= dicsiz;
             dd->count++;
           }
@@ -1510,7 +1529,11 @@ static int32 LhA_Decrunch(FILE *in, FILE *out, int size, uint32 Method)
             dd->count += c;
             while(c--)
             {
-              text[dd->loc++] = fputc(text[i++ & dicsiz], out);
+              int res = fputc(text[i++ & dicsiz], out);
+              if (res < 0) {
+                goto error;
+              }        
+              text[dd->loc++] = res;
               dd->loc &= dicsiz;
             }
           }
@@ -1526,6 +1549,11 @@ static int32 LhA_Decrunch(FILE *in, FILE *out, int size, uint32 Method)
   else
     err = -1;
   return err;
+
+error:
+  free(dd->text);
+  free(dd);
+  return -1;
 }
 
 /*
@@ -1678,6 +1706,7 @@ static int get_header(FILE *f, struct lha_data *data)
 {
 	uint8 buf[21];
 	int size, level, namelen;
+	int error;
 
 	memset(data, 0, sizeof(struct lha_data));
 	if (fread(buf, 1, 21, f) != 21)
@@ -1690,22 +1719,47 @@ static int get_header(FILE *f, struct lha_data *data)
 		data->method = readmem32b(buf + 2);
 		data->packed_size = readmem32l(buf + 7);
 		data->original_size = readmem32l(buf + 11);
-		namelen = read8(f);
-		fread(data->name, 1, namelen, f);
-		data->crc = read16l(f);
-		fseek(f, size + 2 - 24 - namelen, SEEK_CUR);
+		namelen = read8(f, &error);
+		if (error != 0) {
+			return -1;
+		}
+		if (fread(data->name, 1, namelen, f) != namelen) {
+			return -1;
+		}
+		data->crc = read16l(f, &error);
+		if (error != 0) {
+			return -1;
+		}
+		if (fseek(f, size + 2 - 24 - namelen, SEEK_CUR) < 0) {
+			return -1;
+		}
 		break;
 	case 1:
 		size = buf[0];
 		data->method = readmem32b(buf + 2);
 		data->packed_size = readmem32l(buf + 7);
 		data->original_size = readmem32l(buf + 11);
-		namelen = read8(f);
-		fread(data->name, 1, namelen, f);
-		data->crc = read16l(f);
-		fseek(f, size - (22 + namelen) - 2, SEEK_CUR);
-		while ((size = read16l(f)) != 0) {
-			fseek(f, size - 2, SEEK_CUR);
+		namelen = read8(f, &error);
+		if (error != 0) {
+			return -1;
+		}
+		if (fread(data->name, 1, namelen, f) != namelen) {
+			return -1;
+		}
+		data->crc = read16l(f, &error);
+		if (error != 0) {
+			return -1;
+		}
+		if (fseek(f, size - (22 + namelen) - 2, SEEK_CUR) < 0) {
+			return -1;
+		}
+		while ((size = read16l(f, &error)) != 0) {
+			if (error != 0) {
+				return -1;
+			}
+			if (fseek(f, size - 2, SEEK_CUR) < 0) {
+				return -1;
+			}
 			data->packed_size -= size;
 		}
 		break;
@@ -1716,19 +1770,36 @@ static int get_header(FILE *f, struct lha_data *data)
 		data->method = readmem32b(buf + 2);
 		data->packed_size = readmem32l(buf + 7);
 		data->original_size = readmem32l(buf + 11);
-		data->crc = read16l(f);
-		read8(f);		/* skip OS id */
-		while ((size = read16l(f)) != 0) {
-			int type = read8(f);
+		data->crc = read16l(f, &error);
+		if (error != 0) {
+			return -1;
+		}
+		read8(f, &error);		/* skip OS id */
+		if (error != 0) {
+			return -1;
+		}
+		while ((size = read16l(f, &error)) != 0) {
+			int type;
 			int s = size - 3;
+			if (error != 0) {
+				return -1;
+			}
+			type = read8(f, &error);
+			if (error != 0) {
+				return -1;
+			}
 			if (type == 0x01) {
 				/* Sanity check */
-				if (s < 0 || s > 256)
+				if (s < 0 || s > 256) {
 					return -1;
-
-				fread(data->name, 1, s, f);
+				}
+				if (fread(data->name, 1, s, f) != s) {
+					return -1;
+				}
 			} else {
-				fseek(f, s, SEEK_CUR);
+				if (fseek(f, s, SEEK_CUR) < 0) {
+					return -1;
+				}
 			}
 		}
 		break;
@@ -1761,7 +1832,9 @@ static int decrunch_lha(FILE *in, FILE *out)
 #endif
 
 		if (exclude_match(data.name)) {
-			fseek(in, data.packed_size, SEEK_CUR);
+			if (fseek(in, data.packed_size, SEEK_CUR) < 0) {
+				return -1;
+			}
 			continue;
 		}
 		return LhA_Decrunch(in, out, data.original_size, data.method);
