@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2015 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -86,7 +86,6 @@ static void do_toneporta(struct module_data *m,
 	xc->porta.dir = xc->period < xc->porta.target ? 1 : -1;
 }
 
-
 void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 		struct xmp_event *e, int fnum)
 {
@@ -388,8 +387,12 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 			break;
 		case EX_FINETUNE:	/* Set finetune */
 			fxp <<= 4;
-			if (!HAS_QUIRK(QUIRK_FT2BUGS) || note > 0)
+			if (HAS_QUIRK(QUIRK_PROTRACK)) {
+				fxp -= 0x80;
+			}
+			if (!HAS_QUIRK(QUIRK_FT2BUGS) || note > 0) {
 				goto fx_finetune;
+			}
 			break;
 		case EX_PATTERN_LOOP:	/* Loop pattern */
 			if (fxp == 0) {
@@ -489,7 +492,9 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 		break;
 	case FX_PATT_DELAY:
 	    fx_patt_delay:
-		p->flow.delay = fxp;
+		if (m->read_event_type != READ_EVENT_ST3 || !p->flow.delay) {
+			p->flow.delay = fxp;
+		}
 		break;
 
 	case FX_S3M_SPEED:	/* Set S3M speed */
@@ -662,13 +667,21 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 		}
 		SET(RETRIG);
 		break;
-	case FX_TREMOR:		/* Tremor */
+	case FX_TREMOR:			/* Tremor */
 		EFFECT_MEMORY(fxp, xc->tremor.memory);
-		if (MSN(fxp) == 0)
-			fxp |= 0x10;
-		if (LSN(fxp) == 0)
-			fxp |= 0x01;
-		xc->tremor.val = fxp;
+		xc->tremor.up = MSN(fxp);
+		xc->tremor.down = LSN(fxp);
+		if (m->read_event_type == READ_EVENT_FT2) {
+			xc->tremor.count |= 0x80;
+		} else {
+			if (xc->tremor.up == 0) {
+				xc->tremor.up++;
+			}
+			if (xc->tremor.down == 0) {
+				xc->tremor.down++;
+			}
+		}
+		SET(TREMOR);
 		break;
 	case FX_XF_PORTA:	/* Extra fine portamento */
 	      fx_xf_porta:
@@ -799,6 +812,43 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 #endif
 
 #ifndef LIBXMP_CORE_PLAYER
+
+	/* SFX effects */
+	case FX_VOL_ADD:
+		if (!IS_VALID_INSTRUMENT(xc->ins)) {
+			break;
+		}
+		SET(NEW_VOL);
+		xc->volume = m->mod.xxi[xc->ins].sub[0].vol + fxp;
+		if (xc->volume > m->volbase) {
+			xc->volume = m->volbase;
+		}
+		break;
+	case FX_VOL_SUB:
+		if (!IS_VALID_INSTRUMENT(xc->ins)) {
+			break;
+		}
+		SET(NEW_VOL);
+		xc->volume = m->mod.xxi[xc->ins].sub[0].vol - fxp;
+		if (xc->volume < 0) {
+			xc->volume =0;
+		}
+		break;
+	case FX_PITCH_ADD:
+		SET_PER(TONEPORTA);
+		xc->porta.target = note_to_period(note - 1, xc->finetune, 0, 0)
+			+ fxp;
+		xc->porta.slide = 2;
+		xc->porta.dir = 1;
+		break;
+	case FX_PITCH_SUB:
+		SET_PER(TONEPORTA);
+		xc->porta.target = note_to_period(note - 1, xc->finetune, 0, 0)
+			- fxp;
+		xc->porta.slide = 2;
+		xc->porta.dir = -1;
+		break;
+
 	/* Saga Musix says:
 	 *
 	 * "When both nibbles of an Fxx command are set, SoundTracker 2.6
