@@ -7,7 +7,7 @@ int new_channel_extras(struct context_data *, struct channel_data *);
 void release_channel_extras(struct context_data *, struct channel_data *);
 void reset_channel_extras(struct context_data *, struct channel_data *);
 
-void play_extras(struct context_data *, struct channel_data *, int, int);
+void play_extras(struct context_data *, struct channel_data *, int);
 
 int extras_get_volume(struct context_data *, struct channel_data *);
 int extras_get_period(struct context_data *, struct channel_data *);
