@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2015 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -130,10 +130,10 @@ static int decrunch(HIO_HANDLE **h, char *filename, char **temp)
 	*temp = NULL;
 	f = (*h)->handle.file;
 
-	fseek(f, 0, SEEK_SET);
 	headersize = fread(b, 1, 1024, f);
-	if (headersize < 100)	/* minimum valid file size */
+	if (headersize < 100) {	/* minimum valid file size */
 		return 0;
+	}
 
 	/* Check built-in depackers */
 	for (i = 0; depacker_list[i] != NULL; i++) {
@@ -162,7 +162,9 @@ static int decrunch(HIO_HANDLE **h, char *filename, char **temp)
 		}
 	}
 
-	fseek(f, 0, SEEK_SET);
+	if (fseek(f, 0, SEEK_SET) < 0) {
+		goto err;
+	}
 
 	if (depacker == NULL && cmd == NULL) {
 		D_(D_INFO "Not packed");
@@ -178,33 +180,41 @@ static int decrunch(HIO_HANDLE **h, char *filename, char **temp)
 
 	D_(D_WARN "Depacking file... ");
 
-	if ((t = make_temp_file(temp)) == NULL)
-		return -1;
+	if ((t = make_temp_file(temp)) == NULL) {
+		goto err;
+	}
 
 	/* Depack file */
 	if (cmd) {
 		D_(D_INFO "External depacker: %s", cmd);
 		if (execute_command(cmd, filename, t) < 0) {
 			D_(D_CRIT "failed");
-			fclose(t);
-			return -1;
+			goto err2;
 		}
 	} else if (depacker) {
 		D_(D_INFO "Internal depacker");
 		if (depacker->depack(f, t) < 0) {
 			D_(D_CRIT "failed");
-			fclose(t);
-			return -1;
+			goto err2;
 		}
 	}
 
 	D_(D_INFO "done");
 
-	fseek(t, 0, SEEK_SET);
+	if (fseek(t, 0, SEEK_SET) < 0) {
+		D_(D_CRIT "fseek error");
+		goto err2;
+	}
+
 	hio_close(*h);
 	*h = hio_open_file(t);
 
 	return res;
+
+    err2:
+	fclose(t);
+    err:
+	return -1;
 }
 
 static void set_md5sum(HIO_HANDLE *f, unsigned char *digest)
@@ -322,9 +332,9 @@ int xmp_test_module(char *path, struct xmp_test_info *info)
 #endif
 
 			if (info != NULL && !is_prowizard) {
-				strncpy(info->name, buf, XMP_NAME_SIZE);
+				strncpy(info->name, buf, XMP_NAME_SIZE - 1);
 				strncpy(info->type, format_loader[i]->name,
-							XMP_NAME_SIZE);
+							XMP_NAME_SIZE - 1);
 			}
 			return 0;
 		}
@@ -354,6 +364,11 @@ static int load_module(xmp_context opaque, HIO_HANDLE *h)
 	test_result = load_result = -1;
 	for (i = 0; format_loader[i] != NULL; i++) {
 		hio_seek(h, 0, SEEK_SET);
+
+		if (hio_error(h)) {
+			/* reset error flag */
+		}
+
 		D_(D_WARN "test %s", format_loader[i]->name);
 		test_result = format_loader[i]->test(h, NULL, 0);
 		if (test_result == 0) {
@@ -379,12 +394,22 @@ static int load_module(xmp_context opaque, HIO_HANDLE *h)
 		goto err_load;
 	}
 
-	/* Sanity check */
+	/* Sanity check: number of channels, module length */
 	if (mod->chn > XMP_MAX_CHANNELS || mod->len > XMP_MAX_MOD_LENGTH) {
 		goto err_load;
 	}
 
-	/* Sanity check */
+	/* Sanity check: channel pan */
+	for (i = 0; i < mod->chn; i++) {
+		if (mod->xxc[i].vol < 0 || mod->xxc[i].vol > 0xff) {
+			goto err_load;
+		}
+		if (mod->xxc[i].pan < 0 || mod->xxc[i].pan > 0xff) {
+			goto err_load;
+		}
+	}
+
+	/* Sanity check: patterns */
 	if (mod->xxp == NULL) {
 		goto err_load;
 	}
@@ -434,8 +459,9 @@ int xmp_load_module(xmp_context opaque, char *path)
 
 	D_(D_WARN "path = %s", path);
 
-	if (stat(path, &st) < 0)
+	if (stat(path, &st) < 0) {
 		return -XMP_ERROR_SYSTEM;
+	}
 
 #ifndef _MSC_VER
 	if (S_ISDIR(st.st_mode)) {
@@ -444,19 +470,21 @@ int xmp_load_module(xmp_context opaque, char *path)
 	}
 #endif
 
-	if ((h = hio_open(path, "rb")) == NULL)
+	if ((h = hio_open(path, "rb")) == NULL) {
 		return -XMP_ERROR_SYSTEM;
+	}
 
 #ifndef LIBXMP_CORE_PLAYER
 	D_(D_INFO "decrunch");
-	if (decrunch(&h, path, &temp_name) < 0)
-		goto err_depack;
+	if (decrunch(&h, path, &temp_name) < 0) {
+		ret = -XMP_ERROR_DEPACK;
+		goto err;
+	}
 
 	size = hio_size(h);
 	if (size < 256) {		/* get size after decrunch */
-		hio_close(h);
-		unlink_temp_file(temp_name);
-		return -XMP_ERROR_FORMAT;
+		ret = -XMP_ERROR_FORMAT;
+		goto err;
 	}
 #endif
 
@@ -465,12 +493,16 @@ int xmp_load_module(xmp_context opaque, char *path)
 
 #ifndef LIBXMP_CORE_PLAYER
 	m->dirname = get_dirname(path);
-	if (m->dirname == NULL)
-		return -XMP_ERROR_SYSTEM;
+	if (m->dirname == NULL) {
+		ret = -XMP_ERROR_SYSTEM;
+		goto err;
+	}
 
 	m->basename = get_basename(path);
-	if (m->basename == NULL)
-		return -XMP_ERROR_SYSTEM;
+	if (m->basename == NULL) {
+		ret = -XMP_ERROR_SYSTEM;
+		goto err;
+	}
 
 	m->filename = path;	/* For ALM, SSMT, etc */
 	m->size = size;
@@ -486,10 +518,10 @@ int xmp_load_module(xmp_context opaque, char *path)
 	return ret;
 
 #ifndef LIBXMP_CORE_PLAYER
-    err_depack:
+    err:
 	hio_close(h);
 	unlink_temp_file(temp_name);
-	return -XMP_ERROR_DEPACK;
+	return ret;
 #endif
 }
 
