@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2015 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -107,7 +107,7 @@ static int ssn_load(struct module_data *m, HIO_HANDLE *f, const int start)
     sfh.nop = hio_read8(f);		/* Number of patterns (0-128) */
 
     /* Sanity check */
-    if (sfh.nop > 128)
+    if (sfh.nos > 64 || sfh.nop > 128)
 	return -1;
 
     sfh.loop = hio_read8(f);		/* Loop order number */
@@ -232,7 +232,7 @@ static int ssn_load(struct module_data *m, HIO_HANDLE *f, const int start)
 		 * command was encountered, there will be no effect (except
 		 * for command 'f', it always changes the speed). 
 		 */
-		if (MSN(ev[2] < 5) && !event->ins)
+		if (MSN(ev[2]) < 5 && !event->ins)
 		    continue;
 
 		event->fxt = fx[MSN(ev[2])];
@@ -244,7 +244,7 @@ static int ssn_load(struct module_data *m, HIO_HANDLE *f, const int start)
 		    event->fxp = LSN(ev[2]);
 		    break;
 		case FX_PER_VIBRATO:
-		    event->fxp = 0x40 || LSN(ev[2]);
+		    event->fxp = 0x40 | LSN(ev[2]);
 		    break;
 		case FX_FINETUNE:
 		    event->fxp = 0x80 + (LSN(ev[2]) << 4);
