@@ -156,7 +156,9 @@ static int read_abk_song(HIO_HANDLE *f, struct abk_song *song, uint32 songs_sect
 
     song_section = hio_read32b(f);
 
-    hio_seek(f, songs_section_offset + song_section, SEEK_SET);
+    if (hio_seek(f, songs_section_offset + song_section, SEEK_SET) < 0) {
+        return -1;
+    }
 
     for (i=0; i<AMOS_ABK_CHANNELS; i++)
     {
@@ -168,7 +170,9 @@ static int read_abk_song(HIO_HANDLE *f, struct abk_song *song, uint32 songs_sect
     /* unused. just progress the file pointer forward */
     (void) hio_read16b(f);
 
-    hio_read(song->song_name, 1, AMOS_STRING_LEN, f);
+    if (hio_read(song->song_name, 1, AMOS_STRING_LEN, f) != AMOS_STRING_LEN) {
+        return -1;
+    }
 
     return 0;
 }
@@ -193,8 +197,10 @@ static int read_abk_pattern(HIO_HANDLE *f, struct xmp_event *events, uint32 patt
     uint16 delay;
     uint16 patdata;
 
-    uint32 storepos;
-    storepos = hio_tell(f);
+    int storepos;
+    if ((storepos = hio_tell(f)) < 0) {
+        return -1;
+    }
 
     /* count how many abk positions are used in this pattern */
     position = 0;
@@ -398,7 +404,10 @@ static struct abk_instrument* read_abk_insts(HIO_HANDLE *f, uint32 inst_section_
             inst[i].sample_length = sampleLength;
         }
     
-        hio_read(inst[i].sample_name, 1, 16, f);
+        if (hio_read(inst[i].sample_name, 1, 16, f) != 16) {
+            free(inst);
+            return NULL;
+	}
     }
 
     return inst;
@@ -478,18 +487,25 @@ static int abk_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
     mod->chn = AMOS_ABK_CHANNELS;
     mod->pat = hio_read16b(f);
+
+    /* Sanity check */
+    if (mod->pat > 256) {
+        return -1;
+    }
+
     mod->trk = mod->chn * mod->pat;
 
     /* move to the start of the instruments section. */
     hio_seek(f, AMOS_MAIN_HEADER + main_header.instruments_offset, SEEK_SET);
     mod->ins = hio_read16b(f);
-    mod->smp = mod->ins;
 
     /* Sanity check */
-    if (mod->pat > 256 || mod->ins > 255) {
+    if (mod->ins > 255) {
 	return -1;
     }
 
+    mod->smp = mod->ins;
+
     /* Read and convert instruments and samples */
 
     if (instrument_init(mod) < 0)
@@ -501,6 +517,9 @@ static int abk_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
     /* read all the instruments in */
     ci = read_abk_insts(f, inst_section_size, mod->ins);
+    if (ci == NULL) {
+        return -1;
+    }
 
     /* store the location of the first sample so we can read them later. */
     first_sample_offset = AMOS_MAIN_HEADER + main_header.instruments_offset + ci[0].sample_offset;
