@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2015 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -21,7 +21,7 @@
  */
 
 /* AMF loader written based on the format specs by Miodrag Vallat with
- * fixes by Andre Timmermans
+ * fixes by Andre Timmermans.
  *
  * The AMF format is the internal format used by DSMI, the DOS Sound and Music
  * Interface, which is the engine of DMP. As DMP was able to play more and more
@@ -87,7 +87,8 @@ static int amf_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	mod->chn = hio_read8(f);
 
 	/* Sanity check */
-	if (mod->ins == 0 || mod->len == 0 || mod->trk == 0 || mod->chn == 0) {
+	if (mod->ins == 0 || mod->len == 0 || mod->trk == 0
+		|| mod->chn == 0 || mod->chn > XMP_MAX_CHANNELS) {
 		return -1;
 	}
 
@@ -159,6 +160,9 @@ static int amf_load(struct module_data *m, HIO_HANDLE *f, const int start)
 		uint8 b;
 		uint32 len, start, end;
 		long pos = hio_tell(f);
+		if (pos < 0) {
+			return -1;
+		}
 		for (i = 0; i < mod->ins; i++) {
 			b = hio_read8(f);
 			if (b != 0 && b != 1) {
@@ -303,13 +307,11 @@ static int amf_load(struct module_data *m, HIO_HANDLE *f, const int start)
 			return -1;
 
 		size = hio_read24l(f);
-/*printf("TRACK %d SIZE %d\n", i, size);*/
 
 		for (j = 0; j < size; j++) {
 			t1 = hio_read8(f);			/* row */
 			t2 = hio_read8(f);			/* type */
 			t3 = hio_read8(f);			/* parameter */
-/*printf("track %d row %d: %02x %02x %02x\n", i, t1, t1, t2, t3);*/
 
 			if (t1 == 0xff && t2 == 0xff && t3 == 0xff)
 				break;
@@ -327,7 +329,7 @@ static int amf_load(struct module_data *m, HIO_HANDLE *f, const int start)
 			} else if (t2 == 0x7f) {	/* copy previous */
 
 				/* Sanity check */
-				if (t1 == 0 || t1 > 256) {
+				if (t1 == 0) {
 					return -1;
 				}
 
