@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2015 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -60,8 +60,9 @@ static int chip_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
 	LOAD_INIT();
 
-	if ((tidx = calloc(1, 1024)) == NULL)
+	if ((tidx = calloc(1, 1024)) == NULL) {
 		goto err;
+	}
 
 	hio_read(&mh.name, 20, 1, f);
 	hio_read16b(f);
@@ -77,6 +78,12 @@ static int chip_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
 	hio_read(&mh.magic, 4, 1, f);
 	mh.len = hio_read8(f);
+
+	/* Sanity check */
+	if (mh.len > 128) {
+		goto err2;
+	}
+
 	mh.restart = hio_read8(f);
 	hio_read(tidx, 1024, 1, f);
 	hio_read16b(f);
@@ -114,7 +121,7 @@ static int chip_load(struct module_data *m, HIO_HANDLE *f, const int start)
 		struct xmp_subinstrument *sub;
 
 		if (subinstrument_alloc(mod, i, 1) < 0)
-			goto err;
+			goto err2;
 
 		sub = &xxi->sub[0];
 
