@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2015 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -107,6 +107,12 @@ static int digi_load(struct module_data *m, HIO_HANDLE *f, const int start)
     hio_read(&dh.unknown, 19, 1, f);
     dh.pat = hio_read8(f);
     dh.len = hio_read8(f);
+
+    /* Sanity check */
+    if (dh.len > 127) {
+        return -1;
+    }
+
     hio_read(&dh.ord, 128, 1, f);
 
     for (i = 0; i < 31; i++)
