@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2015 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -265,12 +265,14 @@ static int get_dait(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 	}
 
 	if (size > 2) {
+		int ret;
+
 		/* Sanity check */
 		if (data->insnum >= mod->ins) {
 			return -1;
 		}
 
-		int ret = load_sample(m, f, SAMPLE_FLAG_BIGEND,
+		ret = load_sample(m, f, SAMPLE_FLAG_BIGEND,
 			&mod->xxs[mod->xxi[data->insnum].sub[0].sid], NULL);
 
 		if (ret < 0)
