@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2015 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -70,7 +70,7 @@ static int get_main(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 	int flags;
 	
 	hio_read(buf, 1, 64, f);
-	strncpy(mod->name, buf, 64);
+	strncpy(mod->name, buf, 63);	/* ensure string terminator */
 	set_type(m, "Galaxy Music System 4.0");
 
 	flags = hio_read8(f);
@@ -97,6 +97,9 @@ static int get_ordr(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 	int i;
 
 	mod->len = hio_read8(f);
+	if (hio_error(f)) {
+		return -1;
+	}
 
 	for (i = 0; i < mod->len; i++)
 		mod->xxo[i] = hio_read8(f);
@@ -228,7 +231,7 @@ static int get_inst(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 	vsw = hio_read8(f);			/* vibrato sweep */
 	hio_read8(f);			/* unknown */
 	hio_read8(f);			/* unknown */
-	vde = hio_read8(f) / 4;		/* vibrato depth */
+	vde = hio_read8(f);		/* vibrato depth */
 	vra = hio_read16l(f) / 16;		/* vibrato speed */
 	hio_read8(f);			/* unknown */
 
@@ -434,8 +437,18 @@ static int gal4_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
 	iff_release(handle);
 
-	for (i = 0; i < mod->chn; i++)
+	/* Alloc missing patterns */
+	for (i = 0; i < mod->pat; i++) {
+		if (mod->xxp[i] == NULL) {
+			if (pattern_tracks_alloc(mod, i, 64) < 0) {
+				return -1;
+			}
+		}
+	}
+
+	for (i = 0; i < mod->chn; i++) {
 		mod->xxc[i].pan = 0x80;
+	}
 
 	m->quirk |= QUIRKS_FT2;
 	m->read_event_type = READ_EVENT_FT2;
