@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2015 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -75,7 +75,7 @@ static int get_init(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 	int flags;
 	
 	hio_read(buf, 1, 64, f);
-	strncpy(mod->name, buf, 64);
+	strncpy(mod->name, buf, 63);	/* ensure string terminator */
 	set_type(m, "Galaxy Music System 5.0");
 	flags = hio_read8(f);	/* bit 0: Amiga period */
 	if (~flags & 0x01)
@@ -362,6 +362,15 @@ static int gal5_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
 	iff_release(handle);
 
+	/* Alloc missing patterns */
+	for (i = 0; i < mod->pat; i++) {
+		if (mod->xxp[i] == NULL) {
+			if (pattern_tracks_alloc(mod, i, 64) < 0) {
+				return -1;
+			}
+		}
+	}
+
 	for (i = 0; i < mod->chn; i++) {
 		mod->xxc[i].pan = data.chn_pan[i] * 2;
 	}
