@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2015 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -118,8 +118,8 @@ static int ims_test(HIO_HANDLE *f, char *t, const int start)
 
     ih.len = hio_read8(f);
     ih.zero = hio_read8(f);
-    hio_read (&ih.orders, 128, 1, f);
-    hio_read (&ih.magic, 4, 1, f);
+    hio_read(&ih.orders, 128, 1, f);
+    hio_read(&ih.magic, 4, 1, f);
   
     if (ih.zero > 1)		/* not sure what this is */
 	return -1;
@@ -157,6 +157,7 @@ static int ims_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
     LOAD_INIT();
 
+    mod->chn = 4;
     mod->ins = 31;
     mod->smp = mod->ins;
     smp_size = 0;
@@ -176,6 +177,9 @@ static int ims_load(struct module_data *m, HIO_HANDLE *f, const int start)
     }
 
     ih.len = hio_read8(f);
+    if (ih.len > 128) {
+        return -1;
+    }
     ih.zero = hio_read8(f);
     hio_read (&ih.orders, 128, 1, f);
     hio_read (&ih.magic, 4, 1, f);
@@ -199,31 +203,41 @@ static int ims_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	return -1;
 
     for (i = 0; i < mod->ins; i++) {
+	struct xmp_instrument *xxi;
+	struct xmp_subinstrument *sub;
+	struct xmp_sample *xxs;
+
 	if (subinstrument_alloc(mod, i, 1) < 0)
 	    return -1;
 
-	mod->xxs[i].len = 2 * ih.ins[i].size;
-	mod->xxs[i].lpe = mod->xxs[i].lps + 2 * ih.ins[i].loop_size;
-	mod->xxs[i].flg = ih.ins[i].loop_size > 1 ? XMP_SAMPLE_LOOP : 0;
-	mod->xxi[i].sub[0].fin = 0; /* ih.ins[i].finetune; */
-	mod->xxi[i].sub[0].vol = ih.ins[i].volume;
-	mod->xxi[i].sub[0].pan = 0x80;
-	mod->xxi[i].sub[0].sid = i;
-	mod->xxi[i].rls = 0xfff;
+	xxi = &mod->xxi[i];
+	sub = &xxi->sub[0];
+	xxs = &mod->xxs[i];
 
-	if (mod->xxs[i].len > 0)
-		mod->xxi[i].nsm = 1;
+	xxs->len = 2 * ih.ins[i].size;
+	xxs->lps = 2 * ih.ins[i].loop_start;
+	xxs->lpe = xxs->lps + 2 * ih.ins[i].loop_size;
+	xxs->flg = ih.ins[i].loop_size > 1 ? XMP_SAMPLE_LOOP : 0;
+	sub->fin = 0; /* ih.ins[i].finetune; */
+	sub->vol = ih.ins[i].volume;
+	sub->pan = 0x80;
+	sub->sid = i;
+	//mod->xxi[i].rls = 0xfff;
+
+	if (xxs->len > 0) {
+		xxi->nsm = 1;
+	}
 
 	instrument_name(mod, i, ih.ins[i].name, 20);
 
 	D_(D_INFO "[%2X] %-20.20s %04x %04x %04x %c V%02x %+d",
-		i, mod->xxi[i].name, mod->xxs[i].len, mod->xxs[i].lps,
-		mod->xxs[i].lpe, ih.ins[i].loop_size > 1 ? 'L' : ' ',
-		mod->xxi[i].sub[0].vol, mod->xxi[i].sub[0].fin >> 4);
+		i, xxi->name, xxs->len, xxs->lps, xxs->lpe,
+		ih.ins[i].loop_size > 1 ? 'L' : ' ', sub->vol, sub->fin >> 4);
     }
 
-    if (pattern_init(mod) < 0)
+    if (pattern_init(mod) < 0) {
 	return -1;
+    }
 
     /* Load and convert patterns */
     D_(D_INFO "Stored patterns: %d", mod->pat);
@@ -234,7 +248,7 @@ static int ims_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
 	for (j = 0; j < 0x100; j++) {
 	    event = &EVENT (i, j & 0x3, j >> 2);
-	    hio_read (ims_event, 1, 3, f);
+	    hio_read(ims_event, 1, 3, f);
 
 	    /* Event format:
 	     *
