@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2015 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -157,6 +157,11 @@ static int mmd1_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	song.mastervol = hio_read8(f);
 	song.numsamples = hio_read8(f);
 
+	/* Sanity check */
+	if (song.numsamples > 63) {
+		return -1;
+	}
+
 	/*
 	 * convert header
 	 */
@@ -289,14 +294,15 @@ static int mmd1_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
 		if (ver > 0) {
 			block.numtracks = hio_read16b(f);
-			block.lines = hio_read16b(f);
+			/* block.lines = */ hio_read16b(f);
 		} else {
 			block.numtracks = hio_read8(f);
-			block.lines = hio_read8(f);
+			/* block.lines = */ hio_read8(f);
 		}
 
-		if (block.numtracks > mod->chn)
+		if (block.numtracks > mod->chn) {
 			mod->chn = block.numtracks;
+		}
 	}
 
 	/* Sanity check */
@@ -429,20 +435,26 @@ static int mmd1_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
 		D_(D_INFO "sample %d smpl_offset = 0x%08x", i, smpl_offset);
 
-		if (smpl_offset == 0)
+		if (smpl_offset == 0) {
 			continue;
-
-		if (hio_seek(f, start + smpl_offset, SEEK_SET) != 0)
+		}
+		if (hio_seek(f, start + smpl_offset, SEEK_SET) < 0) {
 			return -1;
+		}
 		instr.length = hio_read32b(f);
 		instr.type = hio_read16b(f);
 
-		pos = hio_tell(f);
+		if ((pos = hio_tell(f)) < 0) {
+			return -1;
+		}
 
 		if (expdata_offset && i < expdata.i_ext_entries) {
 			struct xmp_instrument *xxi = &mod->xxi[i];
-			if (hio_seek(f, iinfo_offset + i * expdata.i_ext_entrsz, SEEK_SET) != 0)
+			int offset = iinfo_offset + i * expdata.i_ext_entrsz;
+			
+			if (offset < 0 || hio_seek(f, offset, SEEK_SET) < 0) {
 				return -1;
+			}
 			hio_read(&xxi->name, 40, 1, f);
 		}
 
@@ -450,8 +462,11 @@ static int mmd1_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
 		exp_smp.finetune = 0;
 		if (expdata_offset && i < expdata.s_ext_entries) {
-			if (hio_seek(f, expsmp_offset + i * expdata.s_ext_entrsz, SEEK_SET) != 0)
-			  return -1;
+			int offset = expsmp_offset + i * expdata.s_ext_entrsz;
+
+			if (offset < 0 || hio_seek(f, offset, SEEK_SET) < 0) {
+				return -1;
+			}
 			exp_smp.hold = hio_read8(f);
 			exp_smp.decay = hio_read8(f);
 			exp_smp.suppress_midi_off = hio_read8(f);
