@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2015 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -133,7 +133,11 @@ void mmd_xlat_fx(struct xmp_event *event, int bpm_on, int bpmlen, int med_8ch,
 		 * This sets the secondary tempo (the number of timing
 		 * pulses per note). The argument must be from 01 to 20.
 		 */
-		event->fxt = FX_SPEED;
+		if (event->fxp >= 1 && event->fxp <= 20) {
+			event->fxt = FX_SPEED;
+		} else {
+			event->fxt = event->fxp = 0;
+		}
 		break;
 	case 0x0a:
 		/* 0A not mentioned but it's Protracker-compatible */
@@ -738,7 +742,7 @@ void mmd_info_text(HIO_HANDLE *f, struct module_data *m, int offset)
 	type = hio_read16b(f);
 	if (type == 1) {	/* 1 = ASCII */
 		len = hio_read32b(f);
-		if (len != 0) {
+		if (len != 0 && len < 0x7fffffff) {
 			m->comment = malloc(len + 1);
 			if (m->comment == NULL)
 				return;
