@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2015 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -148,29 +148,21 @@ static int mod_test(HIO_HANDLE *f, char *t, const int start)
 
     hio_seek(f, start + 20, SEEK_SET);
     for (i = 0; i < 31; i++) {
+	uint8 x;
+
 	hio_seek(f, 22, SEEK_CUR);		/* Instrument name */
 
 	/* OpenMPT can create mods with large samples */
 	hio_read16b(f);				/* sample size */
-	if (hio_read8(f) & 0xf0)		/* test finetune */
+
+	/* Chris Spiegel tells me that sandman.mod can have 0x20 in finetune */
+	x = hio_read8(f);
+	if (x & 0xf0 && x != 0x20)		/* test finetune */
 		return -1;
 	if (hio_read8(f) > 0x40)		/* test volume */
 		return -1;
 	hio_read16b(f);				/* loop start */
 	hio_read16b(f);				/* loop size */
-
-#if 0
-	if (hio_read16b(f) & 0x8000)		/* test length */
-		return -1;
-	if (hio_read8(f) & 0xf0)		/* test finetune */
-		return -1;
-	if (hio_read8(f) > 0x40)		/* test volume */
-		return -1;
-	if (hio_read16b(f) & 0x8000)		/* test loop start */
-		return -1;
-	if (hio_read16b(f) & 0x8000)		/* test loop size */
-		return -1;
-#endif
     }
 
     /* Test for UNIC tracker modules
@@ -283,7 +275,10 @@ static int mod_load(struct module_data *m, HIO_HANDLE *f, const int start)
     mh.restart = hio_read8(f);
     hio_read(&mh.order, 128, 1, f);
     memset(magic, 0, 8);
-    hio_read(magic, 4, 1, f);
+    hio_read(magic, 1, 4, f);
+    if (hio_error(f)) {
+        return -1;
+    }
 
     for (i = 0; mod_magic[i].ch; i++) {
 	if (!(strncmp (magic, mod_magic[i].magic, 4))) {
@@ -375,6 +370,9 @@ static int mod_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
     if (0x43c + mod->pat * 4 * mod->chn * 0x40 + smp_size < m->size) {
 	int pos = hio_tell(f);
+        if (pos < 0) {
+           return -1;
+        }
 	hio_seek(f, start + 0x43c + mod->pat * 4 * mod->chn * 0x40 + smp_size, SEEK_SET);
 	hio_read(idbuffer, 1, 4, f);
 	hio_seek(f, start + pos, SEEK_SET);
@@ -442,15 +440,15 @@ static int mod_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	mod->rst = mh.restart;
     }
 
-    if (mod->chn != 4 && mh.restart == 0x7f) {
-	tracker_id = TRACKER_SCREAMTRACKER3;
-	m->read_event_type = READ_EVENT_ST3;
-    }
-
-    if (mod->chn == 4 && mh.restart == 0x7f) {
-	if (has_loop_0) {
-	    tracker_id = TRACKER_CLONE;
-	}
+    if (mh.restart == 0x7f) {
+        if (mod->chn == 4) {
+	    if (has_loop_0) {
+	        tracker_id = TRACKER_CLONE;
+	    }
+        } else {
+	    tracker_id = TRACKER_SCREAMTRACKER3;
+	    m->read_event_type = READ_EVENT_ST3;
+        }
     }
 
     if (mh.restart != 0x78 && mh.restart < 0x7f) {
@@ -525,6 +523,10 @@ static int mod_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
 skip_test:
 
+    if (mod->chn >= XMP_MAX_CHANNELS) {
+        return -1;
+    }
+
     mod->trk = mod->chn * mod->pat;
 
     for (i = 0; i < mod->ins; i++) {
@@ -551,6 +553,9 @@ skip_test:
 	    return -1;
 
 	pos = hio_tell(f);
+	if (pos < 0) {
+		return -1;
+	}
 
 	for (j = 0; j < (64 * mod->chn); j++) {
             int period;
@@ -558,16 +563,9 @@ skip_test:
 	    event = &EVENT(i, j % mod->chn, j / mod->chn);
 	    hio_read(mod_event, 1, 4, f);
 
-	    /* Check out-of-range notes in Amiga trackers */
 	    period = ((int)(LSN(mod_event[0])) << 8) | mod_event[1];
 	    if (period != 0 && (period < 108 || period > 907)) {
 		out_of_range = 1;
-		if (tracker_id == TRACKER_PROTRACKER ||
-		    tracker_id == TRACKER_NOISETRACKER ||
-		    tracker_id == TRACKER_PROBABLY_NOISETRACKER ||
-		    tracker_id == TRACKER_SOUNDTRACKER) {   /* note > B-3 */
-			tracker_id = TRACKER_UNKNOWN;
-		}
 	    }
 
 	    /* Filter noisetracker events */
@@ -578,10 +576,23 @@ skip_test:
         	if ((fxt > 0x06 && fxt < 0x0a) || (fxt == 0x0e && fxp > 1)) {
 		    tracker_id = TRACKER_UNKNOWN;
 		}
-
 	    }
 	}
 
+        if (out_of_range) {
+            if (tracker_id == TRACKER_UNKNOWN && mh.restart == 0x7f) {
+	        tracker_id = TRACKER_SCREAMTRACKER3;
+            }
+
+	    /* Check out-of-range notes in Amiga trackers */
+            if (tracker_id == TRACKER_PROTRACKER ||
+                    tracker_id == TRACKER_NOISETRACKER ||
+                    tracker_id == TRACKER_PROBABLY_NOISETRACKER ||
+                    tracker_id == TRACKER_SOUNDTRACKER) {   /* note > B-3 */
+                tracker_id = TRACKER_UNKNOWN;
+            }
+        }
+
 	hio_seek(f, pos, SEEK_SET);
 
 	for (j = 0; j < (64 * mod->chn); j++) {
@@ -684,7 +695,7 @@ skip_test:
 	if (!mod->xxs[i].len)
 	    continue;
 
-	flags = ptkloop ? SAMPLE_FLAG_FULLREP : 0;
+	flags = (ptkloop && mod->xxs[i].lps == 0) ? SAMPLE_FLAG_FULLREP : 0;
 
 	if (ptsong) {
 	    HIO_HANDLE *s;
@@ -700,8 +711,13 @@ skip_test:
 	    }
 	} else {
 	    uint8 buf[5];
-	    long pos = hio_tell(f);
-            int num = hio_read(buf, 1, 5, f);
+            long pos;
+            int num;
+
+	    if ((pos = hio_tell(f)) < 0) {
+                return -1;
+            }
+            num = hio_read(buf, 1, 5, f);
 
 	    if (num == 5 && !memcmp(buf, "ADPCM", 5)) {
 		flags |= SAMPLE_FLAG_ADPCM;
