@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2015 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -89,19 +89,28 @@ static int mtm_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	mfh.patterns = hio_read8(f);	/* Number of patterns saved */
 	mfh.modlen = hio_read8(f);	/* Module length */
 	mfh.extralen = hio_read16l(f);	/* Length of the comment field */
-	mfh.samples = hio_read8(f);	/* Number of samples */
-	mfh.attr = hio_read8(f);	/* Always zero */
-	mfh.rows = hio_read8(f);	/* Number rows per track */
 
-	/* Sanity check */
+	mfh.samples = hio_read8(f);	/* Number of samples */
+	if (mfh.samples > 63) {
+		return -1;
+	}
+
+	mfh.attr = hio_read8(f);	/* Always zero */
+
+	mfh.rows = hio_read8(f);	/* Number rows per track */
 	if (mfh.rows != 64)
 		return -1;
 
 	mfh.channels = hio_read8(f);	/* Number of tracks per pattern */
+	if (mfh.channels > XMP_MAX_CHANNELS) {
+		return -1;
+	}
+
 	hio_read(&mfh.pan, 32, 1, f);	/* Pan positions for each channel */
 
-	if (mfh.channels > XMP_MAX_CHANNELS)
+	if (hio_error(f)) {
 		return -1;
+	}
 
 #if 0
 	if (strncmp((char *)mfh.magic, "MTM", 3))
