@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2015 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -73,12 +73,7 @@ static int no_test(HIO_HANDLE *f, char *t, const int start)
 
 	hio_seek(f, start + 5, SEEK_SET);
 
-	if (nsize > XMP_NAME_SIZE) {
-		read_title(f, t, XMP_NAME_SIZE);
-		hio_seek(f, nsize - XMP_NAME_SIZE, SEEK_CUR);
-	} else {
-		read_title(f, t, nsize);
-	}
+	read_title(f, t, nsize);
 
 	return 0;
 }
@@ -160,6 +155,10 @@ static int no_load(struct module_data *m, HIO_HANDLE *f, const int start)
 			return -1;
 
 		nsize = hio_read8(f);
+		if (hio_error(f)) {
+			return -1;
+		}
+
 		hasname = 0;
 		for (j = 0; j < nsize; j++) {
 			uint8 x = hio_read8(f);
