@@ -31,13 +31,13 @@ static int depack_tp3(HIO_HANDLE *in, FILE *out)
 
 	hio_seek(in, 8, SEEK_CUR);
 	pw_move_data(out, in, 20);		/* title */
-	nins = hio_read16b(in) / 8;			/* number of sample */
+	nins = hio_read16b(in) / 8;		/* number of sample */
 
 	for (i = 0; i < nins; i++) {
 		pw_write_zero(out, 22);		/*sample name */
 
-		c3 = hio_read8(in);			/* read finetune */
-		c4 = hio_read8(in);			/* read volume */
+		c3 = hio_read8(in);		/* read finetune */
+		c4 = hio_read8(in);		/* read volume */
 
 		write16b(out, size = hio_read16b(in)); /* size */
 		ssize += size * 2;
@@ -57,7 +57,13 @@ static int depack_tp3(HIO_HANDLE *in, FILE *out)
 
 	/* read size of pattern table */
 	hio_read8(in);
-	write8(out, len = hio_read8(in));		/* sequence length */
+	write8(out, len = hio_read8(in));	/* sequence length */
+
+	/* Sanity check */
+	if (len >= 128) {
+		return -1;
+	}
+
 	write8(out, 0x7f);			/* ntk byte */
 
 	for (npat = i = 0; i < len; i++) {
@@ -92,13 +98,12 @@ static int depack_tp3(HIO_HANDLE *in, FILE *out)
 
 			hio_seek(in, pat_ofs + trk_ofs[i][j], SEEK_SET);
 
-			for (k = 0; k < 64; k++) {
+			for (k = 0; k >= 0 && k < 64; k++) {
 				uint8 *p = pdata + k * 16 + j * 4;
 
 				c1 = hio_read8(in);
 				if ((c1 & 0xc0) == 0xc0) {
-					k += (0x100 - c1);
-					k -= 1;
+					k += 0x100 - c1 - 1;
 					continue;
 				}
 
@@ -124,10 +129,15 @@ static int depack_tp3(HIO_HANDLE *in, FILE *out)
 
 				ins = ((c2 >> 4) & 0x0f) | ((c1 >> 2) & 0x10);
 
-				if ((c1 & 0x40) == 0x40)
+				if ((c1 & 0x40) == 0x40) {
 					note = 0x7f - c1;
-				else
+				} else {
 					note = c1 & 0x3f;
+				}
+
+				if (note >= 37) {
+					return -1;
+				}
 
 				fxt = c2 & 0x0f;
 
@@ -159,8 +169,12 @@ static int depack_tp3(HIO_HANDLE *in, FILE *out)
 				p[3] = fxp;
 			}
 			where = hio_tell(in);
-			if (where > max_trk_ofs)
+			if (where < 0) {
+				return -1;
+			}
+			if (where > max_trk_ofs) {
 				max_trk_ofs = where;
+			}
 		}
 		fwrite(pdata, 1024, 1, out);
 	}
