@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2015 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -63,7 +63,7 @@ static int psm_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	hio_read32b(f);
 
 	hio_read(buf, 1, 60, f);
-	strncpy(mod->name, (char *)buf, XMP_NAME_SIZE);
+	strncpy(mod->name, (char *)buf, 60);
 
 	type = hio_read8(f);		/* song type */
 	ver = hio_read8(f);		/* song version */
@@ -121,6 +121,7 @@ static int psm_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
 		hio_read(buf, 1, 13, f);	/* sample filename */
 		hio_read(buf, 1, 24, f);	/* sample description */
+		buf[24] = 0;			/* add string termination */
 		strncpy((char *)mod->xxi[i].name, (char *)buf, 24);
 		adjust_string((char *)mod->xxi[i].name);
 		p_smp[i] = hio_read32l(f);
@@ -162,7 +163,13 @@ static int psm_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
 		len = hio_read16l(f) - 4;
 		rows = hio_read8(f);
+		if (rows > 64) {
+			return -1;
+		}
 		chan = hio_read8(f);
+		if (chan > 32) {
+			return -1;
+		}
 
 		if (pattern_tracks_alloc(mod, i, rows) < 0)
 			return -1;
