@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2015 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -131,8 +131,8 @@ static int ptm_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	return -1;
 
     /* Sanity check */
-    if (pfh.ordnum > 256 || pfh.insnum > 255 || pfh.patnum > 256 ||
-      pfh.chnnum > XMP_MAX_CHANNELS) {
+    if (pfh.ordnum > 256 || pfh.insnum > 255 || pfh.patnum > 128 ||
+      pfh.chnnum > 32) {
 	return -1;
     }
 
