@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2015 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -98,8 +98,9 @@ static int pw_load(struct module_data *m, HIO_HANDLE *h, const int start)
 
 	/* Prowizard depacking */
 
-	if ((temp = make_temp_file(&temp_name)) == NULL)
+	if ((temp = make_temp_file(&temp_name)) == NULL) {
 		goto err;
+	}
 
 	if (pw_wizardry(h, temp, &name) < 0) {
 		fclose(temp);
@@ -109,11 +110,12 @@ static int pw_load(struct module_data *m, HIO_HANDLE *h, const int start)
 	/* Module loading */
 
 	if ((f = hio_open_file(temp)) == NULL) {
-		fclose(temp);
 		goto err2;
 	}
 
-	LOAD_INIT();
+	if (hio_seek(f, 0, start) < 0) {
+		goto err3;
+	}
 
 	hio_read(&mh.name, 20, 1, f);
 	for (i = 0; i < 31; i++) {
@@ -129,8 +131,9 @@ static int pw_load(struct module_data *m, HIO_HANDLE *h, const int start)
 	hio_read(&mh.order, 128, 1, f);
 	hio_read(&mh.magic, 4, 1, f);
 
-	if (memcmp(mh.magic, "M.K.", 4))
+	if (memcmp(mh.magic, "M.K.", 4)) {
 		goto err3;
+	}
 		
 	mod->ins = 31;
 	mod->smp = mod->ins;
@@ -152,8 +155,9 @@ static int pw_load(struct module_data *m, HIO_HANDLE *h, const int start)
 	snprintf(mod->type, XMP_NAME_SIZE, "%s", name);
 	MODULE_INFO();
 
-	if (instrument_init(mod) < 0)
+	if (instrument_init(mod) < 0) {
 		goto err3;
+	}
 
 	for (i = 0; i < mod->ins; i++) {
 		if (subinstrument_alloc(mod, i, 1) < 0)
@@ -182,8 +186,9 @@ static int pw_load(struct module_data *m, HIO_HANDLE *h, const int start)
 			     mod->xxi[i].sub[0].fin >> 4);
 	}
 
-	if (pattern_init(mod) < 0)
+	if (pattern_init(mod) < 0) {
 		goto err3;
+	}
 
 	/* Load and convert patterns */
 	D_(D_INFO "Stored patterns: %d", mod->pat);
