@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2015 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -333,15 +333,19 @@ static int stx_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	    }
 
 	    if (b & S3M_FX_FOLLOWS) {
-		event->fxt = fx[hio_read8(f)];
-		event->fxp = hio_read8(f);
-		switch (event->fxt) {
-		case FX_SPEED:
-		    event->fxp = MSN (event->fxp);
-		    break;
-		case FX_NONE:
-		    event->fxp = event->fxt = 0;
-		    break;
+		int t = hio_read8(f);
+		int p = hio_read8(f);
+		if (t <= 10) {
+		    event->fxt = fx[t];
+		    event->fxp = p;
+		    switch (event->fxt) {
+		    case FX_SPEED:
+		        event->fxp = MSN(event->fxp);
+		        break;
+		    case FX_NONE:
+		        event->fxp = event->fxt = 0;
+		        break;
+		    }
 		}
 	    }
 	}
