@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2015 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -91,6 +91,9 @@ static int load_patterns(struct module_data *m, int version, HIO_HANDLE *f)
 
 	xph.datasize = hio_read16l(f);
 	hio_seek(f, xph.length - headsize, SEEK_CUR);
+	if (hio_error(f)) {
+            goto err;
+	}
 
 	r = xph.rows;
 	if (r == 0)
@@ -402,12 +405,12 @@ static int load_instruments(struct module_data *m, int version, HIO_HANDLE *f)
 		xxi->pei.lpe = xi.p_end;
 		xxi->pei.flg = xi.p_type;
 
-		if (xxi->aei.npt <= 0 || xxi->aei.npt > XMP_MAX_ENV_POINTS)
+		if (xxi->aei.npt <= 0 || xxi->aei.npt > 12 /*XMP_MAX_ENV_POINTS*/)
 		    xxi->aei.flg &= ~XMP_ENVELOPE_ON;
 		else
 		    memcpy(xxi->aei.data, xi.v_env, xxi->aei.npt * 4);
 
-		if (xxi->pei.npt <= 0 || xxi->pei.npt > XMP_MAX_ENV_POINTS)
+		if (xxi->pei.npt <= 0 || xxi->pei.npt > 12 /*XMP_MAX_ENV_POINTS*/)
 		    xxi->pei.flg &= ~XMP_ENVELOPE_ON;
 		else
 		    memcpy(xxi->pei.data, xi.p_env, xxi->pei.npt * 4);
@@ -451,7 +454,7 @@ static int load_instruments(struct module_data *m, int version, HIO_HANDLE *f)
 		sub->xpo = xsh[j].relnote;
 		sub->fin = xsh[j].finetune;
 		sub->vwf = xi.y_wave;
-		sub->vde = xi.y_depth;
+		sub->vde = xi.y_depth << 2;
 		sub->vra = xi.y_rate;
 		sub->vsw = xi.y_sweep;
 		sub->sid = sample_num;
