@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2015 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -293,7 +293,7 @@ void med_play_extras(struct context_data *ctx, struct channel_data *xc, int chn)
 			while (WT != 0xfd) ;
 			break;
 		case 0xfa:	/* JVS */
-			jws = WT;
+			jvs = WT;
 			break;
 		case 0xf7:	/* VWF */
 			ce->vwf = WT;
