@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2015 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -112,17 +112,18 @@ static void update_invloop(struct module_data *m, struct channel_data *xc)
 static int is_first_frame(struct context_data *ctx)
 {
 	struct player_data *p = &ctx->p;
-#ifndef LIBXMP_CORE_DISABLE_IT
 	struct module_data *m = &ctx->m;
 
-	if (m->read_event_type == READ_EVENT_IT) {
+	switch (m->read_event_type) {
+#ifndef LIBXMP_CORE_DISABLE_IT
+	case READ_EVENT_IT:
+		/* fall through */
+#endif
+	case READ_EVENT_ST3:
 		return p->frame % p->speed == 0;
-	} else {
+	default:
 		return p->frame == 0;
 	}
-#else
-	return p->frame == 0;
-#endif
 }
 
 static void reset_channels(struct context_data *ctx)
@@ -132,7 +133,7 @@ static void reset_channels(struct context_data *ctx)
 	struct xmp_module *mod = &m->mod;
 	struct smix_data *smix = &ctx->smix;
 	struct channel_data *xc;
-	int i, j;
+	int i;
 
 #ifndef LIBXMP_CORE_PLAYER
 	m->synth->reset(ctx);
@@ -163,7 +164,6 @@ static void reset_channels(struct context_data *ctx)
 
 	for (i = 0; i < p->virt.num_tracks; i++) {
 		xc = &p->xc_data[i];
-		xc->filter.cutoff = 0xff;
 
 		if (i >= mod->chn && i < mod->chn + smix->chn) {
 			xc->mastervol = 0x40;
@@ -173,8 +173,13 @@ static void reset_channels(struct context_data *ctx)
 			xc->pan.val = mod->xxc[i].pan;
 		}
 		
+#ifndef LIBXMP_CORE_DISABLE_IT
+		xc->filter.cutoff = 0xff;
+
 		/* Amiga split channel */
 		if (mod->xxc[i].flg & XMP_CHANNEL_SPLIT) {
+			int j;
+
 			xc->split = ((mod->xxc[i].flg & 0x30) >> 4) + 1;
 			/* Connect split channel pairs */
 			for (j = 0; j < i; j++) {
@@ -188,6 +193,7 @@ static void reset_channels(struct context_data *ctx)
 		} else {
 			xc->split = 0;
 		}
+#endif
 
 		/* Surround channel */
 		if (mod->xxc[i].flg & XMP_CHANNEL_SURROUND) {
@@ -289,8 +295,10 @@ static inline void read_row(struct context_data *ctx, int pat, int row)
 #endif
 			}
 		} else {
-			/* Reset flags. See SlideDelay.it */
-			p->xc_data[chn].flags = 0;
+			if (m->read_event_type == READ_EVENT_IT) {
+				/* Reset flags. See SlideDelay.it */
+				p->xc_data[chn].flags = 0;
+			}
 		}
 	}
 }
@@ -316,6 +324,56 @@ static inline int get_channel_vol(struct context_data *ctx, int chn)
 	return p->channel_vol[root];
 }
 
+static int tremor_ft2(struct context_data *ctx, int chn, int finalvol)
+{
+	struct player_data *p = &ctx->p;
+	struct channel_data *xc = &p->xc_data[chn];
+
+	if (xc->tremor.count & 0x80) {
+		if (TEST(TREMOR) && p->frame != 0) {
+			xc->tremor.count &= ~0x20;
+			if (xc->tremor.count == 0x80) {
+				/* end of down cycle, set up counter for up  */
+				xc->tremor.count = xc->tremor.up | 0xc0;
+			} else if (xc->tremor.count == 0xc0) {
+				/* end of up cycle, set up counter for down */
+				xc->tremor.count = xc->tremor.down | 0x80;
+			} else {
+				xc->tremor.count--;
+			}
+		}
+
+		if ((xc->tremor.count & 0xe0) == 0x80) {
+			finalvol = 0;
+		}
+	}
+
+	return finalvol;
+}
+
+static int tremor_s3m(struct context_data *ctx, int chn, int finalvol)
+{
+	struct player_data *p = &ctx->p;
+	struct channel_data *xc = &p->xc_data[chn];
+
+	if (TEST(TREMOR)) {
+		if (xc->tremor.count == 0) {
+			/* end of down cycle, set up counter for up  */
+			xc->tremor.count = xc->tremor.up | 0x80;
+		} else if (xc->tremor.count == 0x80) {
+			/* end of up cycle, set up counter for down */
+			xc->tremor.count = xc->tremor.down;
+		}
+
+		xc->tremor.count--;
+
+		if (~xc->tremor.count & 0x80) {
+			finalvol = 0;
+		}
+	}
+
+	return finalvol;
+}
 
 /*
  * Update channel data
@@ -332,6 +390,7 @@ static void process_volume(struct context_data *ctx, int chn, int act)
 	int finalvol;
 	uint16 vol_envelope;
 	int gvol;
+	int fade = 0;
 
 	instrument = get_instrument(ctx, xc->ins);
 
@@ -341,13 +400,33 @@ static void process_volume(struct context_data *ctx, int chn, int act)
 	 * In XM it depends on envelope (see graff-strange_land.xm vs
 	 * Decibelter - Cosmic 'Wegian Mamas.xm)
 	 */
-	if (!HAS_QUIRK(QUIRK_KEYOFF)) {
-		if (TEST_NOTE(NOTE_RELEASE) && !(instrument->aei.flg & XMP_ENVELOPE_ON))
-			xc->fadeout = 0;
+	if (HAS_QUIRK(QUIRK_KEYOFF)) {
+		/* If IT, only apply fadeout on note release if we don't
+		 * have envelope, or if we have envelope loop
+		 */
+		if (TEST_NOTE(NOTE_RELEASE) || act == VIRT_ACTION_OFF) {
+			if ((~instrument->aei.flg & XMP_ENVELOPE_ON) ||
+			    (instrument->aei.flg & XMP_ENVELOPE_LOOP)) {
+				fade = 1;
+			}
+		}
+	} else {
+		if (~instrument->aei.flg & XMP_ENVELOPE_ON) {
+			if (TEST_NOTE(NOTE_RELEASE)) {
+				xc->fadeout = 0;
+			}
+		}
+
+		if (TEST_NOTE(NOTE_RELEASE) || act == VIRT_ACTION_OFF) {
+			fade = 1;
+		}
 	}
 
-	if (TEST_NOTE(NOTE_FADEOUT | NOTE_RELEASE) || act == VIRT_ACTION_FADE
-	    || act == VIRT_ACTION_OFF) {
+	if (TEST_NOTE(NOTE_FADEOUT) || act == VIRT_ACTION_FADE) {
+		fade = 1;
+	}
+
+	if (fade) {
 		if (xc->fadeout > xc->ins_fade) {
 			xc->fadeout -= xc->ins_fade;
 		} else {
@@ -397,7 +476,7 @@ static void process_volume(struct context_data *ctx, int chn, int act)
 #endif
 
 	if (TEST(TREMOLO)) {
-		finalvol += get_lfo(ctx, &xc->tremolo.lfo, 1 << 6, 0);
+		finalvol += get_lfo(ctx, &xc->tremolo.lfo, 0) / (1 << 6);
 		if (!is_first_frame(ctx) || HAS_QUIRK(QUIRK_VIBALL)) {
 			update_lfo(&xc->tremolo.lfo);
 		}
@@ -432,26 +511,17 @@ static void process_volume(struct context_data *ctx, int chn, int act)
 		finalvol = (finalvol * instrument->vol * xc->gvl) >> 12;
 	}
 
-	if (xc->tremor.val) {
-		if (xc->tremor.count == 0) {
-			/* end of down cycle, set up counter for up  */
-			xc->tremor.count = MSN(xc->tremor.val) | 0x80;
-		} else if (xc->tremor.count == 0x80) {
-			/* end of up cycle, set up counter for down */
-			xc->tremor.count = LSN(xc->tremor.val);
-		}
-
-		xc->tremor.count--;
-
-		if (~xc->tremor.count & 0x80) {
-			finalvol = 0;
-		}
+	if (m->read_event_type == READ_EVENT_FT2) {
+		finalvol = tremor_ft2(ctx, chn, finalvol);
+	} else {
+		finalvol = tremor_s3m(ctx, chn, finalvol);
 	}
 
-	if (chn < m->mod.chn)
+	if (chn < m->mod.chn) {
 		finalvol = finalvol * p->master_vol / 100;
-	else
+	} else {
 		finalvol = finalvol * p->smix_vol / 100;
+	}
 
 	xc->info_finalvol = TEST_NOTE(NOTE_SAMPLE_END) ? 0 : finalvol;
 
@@ -472,10 +542,10 @@ static void process_frequency(struct context_data *ctx, int chn, int act)
 	struct module_data *m = &ctx->m;
 	struct channel_data *xc = &p->xc_data[chn];
 	struct xmp_instrument *instrument;
-	double period;
+	double period, vibrato;
 	int linear_bend;
 	int frq_envelope;
-	int arp, vibrato;
+	int arp;
 #ifndef LIBXMP_CORE_DISABLE_IT
 	int cutoff, resonance;
 #endif
@@ -506,7 +576,8 @@ static void process_frequency(struct context_data *ctx, int chn, int act)
 #endif
 
 	/* Instrument vibrato */
-	vibrato = get_lfo(ctx, &xc->insvib.lfo, (1024 * (1 + xc->insvib.sweep)), 1);
+	vibrato = 1.0 * get_lfo(ctx, &xc->insvib.lfo, 1) /
+				(4096 * (1 + xc->insvib.sweep));
 	update_lfo(&xc->insvib.lfo);
 	if (xc->insvib.sweep > 1) {
 		xc->insvib.sweep -= 2;
@@ -517,7 +588,7 @@ static void process_frequency(struct context_data *ctx, int chn, int act)
 	/* Vibrato */
 	if (TEST(VIBRATO) || TEST_PER(VIBRATO)) {
 		int shift = HAS_QUIRK(QUIRK_VIBHALF) ? 10 : 9;
-		int vib = get_lfo(ctx, &xc->vibrato.lfo, 1 << shift, 1);
+		int vib = get_lfo(ctx, &xc->vibrato.lfo, 1) / (1 << shift);
 
 		if (HAS_QUIRK(QUIRK_VIBINV)) {
 			vibrato -= vib;
@@ -535,8 +606,15 @@ static void process_frequency(struct context_data *ctx, int chn, int act)
 	period += extras_get_period(ctx, xc);
 #endif
 
+	/* Arpeggio */
+
+	arp = xc->arpeggio.val[xc->arpeggio.count];
+
+	/* Pitch bend */
+
 	linear_bend = period_to_bend(period + vibrato, xc->note,
-			TEST(GLISSANDO), HAS_QUIRK(QUIRK_LINEAR), xc->per_adj);
+			TEST(GLISSANDO) || (HAS_QUIRK(QUIRK_FT2BUGS) && arp),
+			HAS_QUIRK(QUIRK_LINEAR), xc->per_adj);
 
 	/* Envelope */
 
@@ -547,13 +625,29 @@ static void process_frequency(struct context_data *ctx, int chn, int act)
 		linear_bend += frq_envelope << 7;
 	}
 
-	/* Arpeggio */
-
-	arp = xc->arpeggio.val[xc->arpeggio.count];
+	/* OpenMPT ArpeggioClamp.xm */
+	if (HAS_QUIRK(QUIRK_FT2BUGS)) {
+		if (xc->note + arp > 107) {
+			if (xc->arpeggio.count > 0) {
+				arp = 108 - xc->note;
+			}
+		}
+	}
+	
 	if (arp != 0) {
 		linear_bend += (100 << 7) * arp;
+
+		/* OpenMPT ArpWrapAround.mod */
+		if (HAS_QUIRK(QUIRK_PROTRACK)) {
+			if (xc->note + arp > 84) {
+				linear_bend -= 12800 * (3 * 12);
+			} else if (xc->note + arp > 83) {
+				virt_setvol(ctx, chn, 0);
+			}
+		}
 	}
 
+
 #ifndef LIBXMP_CORE_PLAYER
 	linear_bend += extras_get_linear_bend(ctx, xc);
 #endif
@@ -626,10 +720,12 @@ static void process_pan(struct context_data *ctx, int chn, int act)
 	}
 	pan_envelope = get_envelope(&instrument->pei, xc->p_idx, 32);
 
+#ifndef LIBXMP_CORE_DISABLE_IT
 	if (TEST(PANBRELLO)) {
-		panbrello = get_lfo(ctx, &xc->panbrello.lfo, 512, 0);
+		panbrello = get_lfo(ctx, &xc->panbrello.lfo, 0) / 512;
 		update_lfo(&xc->panbrello.lfo);
 	}
+#endif
 
 	finalpan = xc->pan.val + panbrello + (pan_envelope - 32) *
 				(128 - abs(xc->pan.val - 128)) / 32;
@@ -726,27 +822,36 @@ static void update_frequency(struct context_data *ctx, int chn)
 	if (!is_first_frame(ctx) || HAS_QUIRK(QUIRK_PBALL)) {
 		if (TEST(PITCHBEND) || TEST_PER(PITCHBEND)) {
 			xc->period += xc->freq.slide;
+			if (HAS_QUIRK(QUIRK_PROTRACK)) {
+				xc->porta.target = xc->period;
+			}
 		}
 
 		/* Do tone portamento */
 		if (TEST(TONEPORTA) || TEST_PER(TONEPORTA)) {
-			int end = 0;
-			if (xc->porta.dir > 0) {
-				xc->period += xc->porta.slide;
-				if (xc->period >= xc->porta.target)
-					end = 1;
-			} else {
-				xc->period -= xc->porta.slide;
-				if (xc->period <= xc->porta.target)
-					end = 1;
-			}
+			if (xc->porta.target > 0) {
+				int end = 0;
+				if (xc->porta.dir > 0) {
+					xc->period += xc->porta.slide;
+					if (xc->period >= xc->porta.target)
+						end = 1;
+				} else {
+					xc->period -= xc->porta.slide;
+					if (xc->period <= xc->porta.target)
+						end = 1;
+				}
 
-			if (end) {
-				/* reached end */
-				xc->period = xc->porta.target;
-				xc->porta.dir = 0;
-				RESET(TONEPORTA);
-				RESET_PER(TONEPORTA);
+				if (end) {
+					/* reached end */
+					xc->period = xc->porta.target;
+					xc->porta.dir = 0;
+					RESET(TONEPORTA);
+					RESET_PER(TONEPORTA);
+
+					if (HAS_QUIRK(QUIRK_PROTRACK)) {
+						xc->porta.target = -1;
+					}
+				}
 			}
 		} 
 	}
@@ -816,11 +921,13 @@ static void play_channel(struct context_data *ctx, int chn)
 
 	xc->info_finalvol = 0;
 
+#ifndef LIBXMP_CORE_DISABLE_IT
 	/* IT tempo slide */
 	if (!is_first_frame(ctx) && TEST(TEMPO_SLIDE)) {
 		p->bpm += xc->tempo.slide;
 		CLAMP(p->bpm, 0x20, 0xff);
 	}
+#endif
 
 	/* Do delay */
 	if (xc->delay > 0) {
