@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2015 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -65,7 +65,11 @@ static struct xmp_subinstrument *get_subinstrument(struct context_data *ctx,
 			int mapped = instrument->map[key].ins;
 			if (mapped != 0xff && mapped >= 0 && mapped < instrument->nsm)
 			  	return &instrument->sub[mapped];
-	  }
+		} else {
+			if (mod->xxi[ins].nsm > 0) {
+				return &instrument->sub[0];
+			}
+		}
 	}
 
 	return NULL;
@@ -133,9 +137,12 @@ static void set_effect_defaults(struct context_data *ctx, int note,
 			xxi = &mod->xxi[xc->ins];
 		}
 
-		xc->finetune = sub->fin;
+		if (!HAS_QUIRK(QUIRK_PROTRACK)) {
+			xc->finetune = sub->fin;
+		}
 		xc->gvl = sub->gvl;
 
+#ifndef LIBXMP_CORE_DISABLE_IT
 		if (sub->ifc & 0x80) {
 			xc->filter.cutoff = (sub->ifc - 0x80) * 2;
 		} else if (~xxi->fei.flg & XMP_ENVELOPE_FLT) {
@@ -148,6 +155,7 @@ static void set_effect_defaults(struct context_data *ctx, int note,
 		} /* else {
 			xc->filter.resonance = 0;
 		} */
+#endif
 
 		set_lfo_depth(&xc->insvib.lfo, sub->vde);
 		set_lfo_rate(&xc->insvib.lfo, sub->vra >> 2);
@@ -159,7 +167,7 @@ static void set_effect_defaults(struct context_data *ctx, int note,
 	}
 
 	xc->delay = 0;
-	xc->tremor.val = 0;
+	xc->tremor.up = xc->tremor.down = 0;
 
 	/* Reset arpeggio */
 	xc->arpeggio.val[0] = 0;
@@ -167,6 +175,14 @@ static void set_effect_defaults(struct context_data *ctx, int note,
 	xc->arpeggio.size = 1;
 }
 
+/* From OpenMPT PortaTarget.mod:
+ * "A new note (with no portamento command next to it) does not reset the
+ *  portamento target. That is, if a previous portamento has not finished yet,
+ *  calling 3xx or 5xx after the new note will slide it towards the old target.
+ *  Once the portamento target period is reached, the target is reset. This
+ *  means that if the period is modified by another slide (e.g. 1xx or 2xx),
+ *  a following 3xx will not slide back to the original target."
+ */
 static void set_period(struct context_data *ctx, int note,
 				struct xmp_subinstrument *sub,
 				struct channel_data *xc, int is_toneporta)
@@ -174,10 +190,15 @@ static void set_period(struct context_data *ctx, int note,
 	struct module_data *m = &ctx->m;
 
 	if (sub != NULL && note >= 0) {
-		xc->porta.target = note_to_period(note, xc->finetune,
+		double per = note_to_period(note, xc->finetune,
 				HAS_QUIRK(QUIRK_LINEAR), xc->per_adj);
+
+		if (!HAS_QUIRK(QUIRK_PROTRACK) || (note > 0 && is_toneporta)) {
+			xc->porta.target = per;
+		}
+
 		if (xc->period < 1 || !is_toneporta) {
-			xc->period = xc->porta.target;
+			xc->period = per;
 		}
 	}
 }
@@ -193,7 +214,7 @@ static void set_period_ft2(struct context_data *ctx, int note,
 {
 	struct module_data *m = &ctx->m;
 
-	if ((note > 0 && is_toneporta) || xc->porta.target == 0) {
+	if (note > 0 && is_toneporta) {
 		xc->porta.target = note_to_period(note, xc->finetune,
 				HAS_QUIRK(QUIRK_LINEAR), xc->per_adj);
 	}
@@ -207,6 +228,7 @@ static void set_period_ft2(struct context_data *ctx, int note,
 
 
 #ifndef LIBXMP_CORE_PLAYER
+#define IS_SFX_PITCH(x) ((x) == FX_PITCH_ADD || (x) == FX_PITCH_SUB)
 #define IS_TONEPORTA(x) ((x) == FX_TONEPORTA || (x) == FX_TONE_VSLIDE \
 		|| (x) == FX_PER_TPORTA)
 #else
@@ -249,9 +271,10 @@ static int read_event_mod(struct context_data *ctx, struct xmp_event *e, int chn
 		RESET_NOTE(NOTE_RELEASE|NOTE_FADEOUT);
 
 		if (IS_VALID_INSTRUMENT(ins)) {
+			sub = get_subinstrument(ctx, ins, e->note - 1);
+
 			if (is_toneporta) {
 				/* Get new instrument volume */
-				sub = get_subinstrument(ctx, ins, e->note - 1);
 				if (sub != NULL) {
 					/* Dennis Lindroos: instrument volume
 					 * is not used on split channels
@@ -264,6 +287,12 @@ static int read_event_mod(struct context_data *ctx, struct xmp_event *e, int chn
 			} else {
 				xc->ins = ins;
 				xc->ins_fade = mod->xxi[ins].rls;
+
+				if (sub != NULL) {
+					if (HAS_QUIRK(QUIRK_PROTRACK)) {
+						xc->finetune = sub->fin;
+					}
+				}
 			}
 		} else {
 			new_invalid_ins = 1;
@@ -323,7 +352,16 @@ static int read_event_mod(struct context_data *ctx, struct xmp_event *e, int chn
 	/* Secondary effect handled first */
 	process_fx(ctx, xc, chn, e, 1);
 	process_fx(ctx, xc, chn, e, 0);
-	set_period(ctx, note, sub, xc, is_toneporta);
+
+#ifndef LIBXMP_CORE_PLAYER
+	if (IS_SFX_PITCH(e->fxt)) {
+ 		xc->period = note_to_period(note, xc->finetune,
+                                HAS_QUIRK(QUIRK_LINEAR), xc->per_adj);
+	} else
+#endif
+	{
+		set_period(ctx, note, sub, xc, is_toneporta);
+	}
 
 	if (sub == NULL) {
 		return 0;
@@ -382,6 +420,14 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 
 	memcpy(&ev, e, sizeof (struct xmp_event));
 
+	/* From OpenMPT TremorReset.xm test case:
+	 * "Even if a tremor effect muted the sample on a previous row, volume
+	 *  commands should be able to override this effect."
+	 */
+	if (ev.vol) {
+		xc->tremor.count &= ~0x80;
+	}
+
 	xc->flags = 0;
 	note = -1;
 	key = ev.note;
@@ -468,6 +514,8 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 				key = 0;
 			}
 		}
+
+		xc->tremor.count = 0x20;
 	}
 
 	/* Check note */
@@ -660,9 +708,12 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 		/* From the OpenMPT test cases (3xx-no-old-samp.xm):
 		 * "An offset effect that points beyond the sample end should
 		 *  stop playback on this channel."
+		 *
+		 * ... except in Skale Tracker (and possibly others), so make this a
+		 *  FastTracker2 quirk. See Armada Tanks game.it (actually an XM).
+		 *  Reported by Vladislav Suschikh.
 		 */
-
-		if (xc->offset.val >= mod->xxs[sub->sid].len) {
+		if (HAS_QUIRK(QUIRK_FT2BUGS) && xc->offset.val >= mod->xxs[sub->sid].len) {
 			virt_resetchannel(ctx, chn);
 		} else {
 
@@ -981,9 +1032,14 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 		int set_new_ins = 1;
 
 		/* portamento_after_keyoff.it test case */
-		if (is_release && is_toneporta && !key) {
-			if (HAS_QUIRK(QUIRK_PRENV) || TEST_NOTE(NOTE_SET)) {
-				is_toneporta = 0;
+		if (is_release && !key) {
+			if (is_toneporta) {
+				if (HAS_QUIRK(QUIRK_PRENV) || TEST_NOTE(NOTE_SET)) {
+					is_toneporta = 0;
+					reset_envelopes_carry(ctx, xc);
+				}
+			} else {
+				/* fixes OpenMPT wnoteoff.it */
 				reset_envelopes_carry(ctx, xc);
 			}
 		}
@@ -1450,8 +1506,9 @@ static int read_event_smix(struct context_data *ctx, struct xmp_event *e, int ch
 
 	if (is_smix_ins) {
 		sub = &smix->xxi[xc->ins - mod->ins].sub[0];
-		if (sub == NULL)
+		if (sub == NULL) {
 			return 0;
+		}
 
 		note = xc->key + sub->xpo;
 		smp = sub->sid;
@@ -1465,6 +1522,9 @@ static int read_event_smix(struct context_data *ctx, struct xmp_event *e, int ch
 	} else {
 		transp = mod->xxi[xc->ins].map[xc->key].xpo;
 		sub = get_subinstrument(ctx, xc->ins, xc->key);
+		if (sub == NULL) {
+			return 0;
+		}
 		note = xc->key + sub->xpo + transp;
 		smp = sub->sid;
 		if (mod->xxs[smp].len == 0)
@@ -1478,7 +1538,7 @@ static int read_event_smix(struct context_data *ctx, struct xmp_event *e, int ch
 	set_effect_defaults(ctx, note, sub, xc, 0);
 	set_period(ctx, note, sub, xc, 0);
 
-	if (e->ins && sub != NULL) {
+	if (e->ins) {
 		reset_envelopes(ctx, xc);
 	}
 
