@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2015 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -391,8 +391,10 @@ static int scan_module(struct context_data *ctx, int ep, int chain)
 		    parm = (f1 == FX_EXTENDED) ? p1 : p2;
 
 		    if ((parm >> 4) == EX_PATT_DELAY) {
-			pdelay = parm & 0x0f;
-			frame_count += pdelay * speed;
+			if (m->read_event_type != READ_EVENT_ST3 || !pdelay) {
+			    pdelay = parm & 0x0f;
+			    frame_count += pdelay * speed;
+                        }
 		    }
 
 		    if ((parm >> 4) == EX_PATTERN_LOOP) {
@@ -459,6 +461,15 @@ static int scan_module(struct context_data *ctx, int ep, int chain)
     row = break_row;
 
 end_module:
+
+    /* Sanity check */
+    {
+        pat = mod->xxo[ord];
+        if (pat >= mod->pat || row >= mod->xxp[pat]->rows) {
+            row = 0;
+        }
+    }
+
     p->scan[chain].num = m->scan_cnt[ord][row];
     p->scan[chain].row = row;
     p->scan[chain].ord = ord;
